-- /*******************************************************************************

--   Copyright (c) 2010, 2013 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata
--   All rights reserved.

-- ********************************************************************************

--   Redistribution and use in source and binary forms, with or without 
--   modification, are permitted provided that the following conditions are met:
--     * Redistributions of source code must retain the above copyright
--       notice, this list of conditions and the following disclaimer.
--     * Redistributions in binary form must reproduce the above copyright
--       notice, this list of conditions and the following disclaimer in the
--       documentation and/or other materials provided with the distribution.
--     * Neither the name of the author nor the names of its contributors may
--       be used to endorse or promote products derived from this software
--       without specific prior written permission.

--   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
--   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
--   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
--   DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
--   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
--   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
--   OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
--   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
--   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
--   EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

-- ********************************************************************************

--   La redistribución y el uso en las formas de código fuente y binario, con o sin
--   modificaciones, están permitidos siempre que se cumplan las siguientes condiciones:
--     * Las redistribuciones del código fuente deben conservar el aviso de copyright
--       anterior, esta lista de condiciones y el siguiente descargo de responsabilidad.
--     * Las redistribuciones en formato binario deben reproducir el aviso de copyright
--       anterior, esta lista de condiciones y el siguiente descargo de responsabilidad
--       en la documentación y/u otros materiales suministrados con la distribución.
--     * Ni el nombre de los titulares de derechos de autor ni los nombres de sus
--       colaboradores pueden usarse para apoyar o promocionar productos derivados de
--       este software sin permiso previo y por escrito.

--   ESTE SOFTWARE SE SUMINISTRA POR LIFIA "COMO ESTÁ" Y CUALQUIER GARANTÍA EXPRESA
--   O IMPLÍCITA, INCLUYENDO, PERO NO LIMITADO A, LAS GARANTÍAS IMPLÍCITAS DE
--   COMERCIALIZACIÓN Y APTITUD PARA UN PROPÓSITO DETERMINADO SON RECHAZADAS. EN
--   NINGÚN CASO LIFIA SERÁ RESPONSABLE POR NINGÚN DAÑO DIRECTO, INDIRECTO, INCIDENTAL,
--   ESPECIAL, EJEMPLAR O CONSECUENTE (INCLUYENDO, PERO NO LIMITADO A, LA ADQUISICIÓN
--   DE BIENES O SERVICIOS; LA PÉRDIDA DE USO, DE DATOS O DE BENEFICIOS; O INTERRUPCIÓN
--   DE LA ACTIVIDAD EMPRESARIAL) O POR CUALQUIER TEORÍA DE RESPONSABILIDAD, YA SEA
--   POR CONTRATO, RESPONSABILIDAD ESTRICTA O AGRAVIO (INCLUYENDO NEGLIGENCIA O
--   CUALQUIER OTRA CAUSA) QUE SURJA DE CUALQUIER MANERA DEL USO DE ESTE SOFTWARE,
--   INCLUSO SI SE HA ADVERTIDO DE LA POSIBILIDAD DE TALES DAÑOS.

-- *******************************************************************************/

local l_util = require 'l_util'
local log = require 'log'
local application = require 'application'
local proxy = require 'proxy'
local sandbox = require 'sandbox'

----------------------------------------------------------------------------------
-- Application Manager API
----------------------------------------------------------------------------------

function appmanagerNew()
	manager = {}

	manager._apps = {}

	manager._app_in_control = nil

	surface = canvas.createSurface(0, 0, canvas.size())
	surface:setZIndex(canvas.zapperZIndex())
	surface:autoFlush(true)

	manager._surface = createSurfaceProxy(surface)

	return manager
end

function appmanagerRun( manager )
	local files = util.scanApps()

	local enabled_apps = loadApps()

	for _, file in pairs(files) do
		registerApp(manager, file, l_util.table.has_value(enabled_apps, file))
	end
end

function appmanagerFinalize( manager )
	local str = ''
	for _, app in pairs(manager._apps) do
		if application.is_enable(app) then
			str = str .. app.file
			str = str .. ';'
		end
		application.finalize(app)
	end

	-- Save apps in storage.
	storage.saveStr("appManager.running_apps", str)
end

function appmanagerList( manager, interactive )
	interactive = interactive or false
	local result = {}

	for _, app in pairs(manager._apps) do
		if (not interactive or (interactive and application.is_interactive(app) and application.is_enable(app))) then
			log.debug( "ApplicationManager", "list application="  .. app.name .. " id=" .. app.id)
			table.insert(result, app)
		end
	end
	return result
end

function appmanagerStartApp( manager, appID )
	log.debug( "ApplicationManager", "StartApp", "application " .. appID)
	local app = getApp(manager, appID)
	if app ~= nil then
		return application.start(app)
	end
	return false
end

function appmanagerStopApp( manager, appID )
	log.debug( "ApplicationManager", "StopApp", "application " .. appID)
	local app = getApp(manager, appID)
	if app ~= nil then
		application.stop(app)
	end
end

function appmanagerEnableApp( manager, appID )
	log.debug( "ApplicationManager", "EnableApp", "application " .. appID)
	local app = getApp(manager, appID)
	if app ~= nil then
		return application.enable(app)
	end
	return false
end

function appmanagerDisableApp( manager, appID )
	log.debug( "ApplicationManager", "DisableApp", "application " .. appID)
	local app = getApp(manager, appID)
	if app ~= nil then
		application.disable(app)
	end
end

function appmanagerStartSetupApp( manager, appID )
	log.info( "ApplicationManager", "StartSetupApp", "application " .. appID )
	local app = getApp(manager, appID)
	if app ~= nil then
		application.start_setup(app)
	end
end

function appmanagerStopSetupApp( manager, appID )
	log.info( "ApplicationManager", "StopSetupApp", "application " .. appID )
	local app = getApp(manager, appID)
	if app ~= nil then
		application.stop_setup(app)
	end
end

function appmanagerDispatchEvent( manager, evt )
	local result = false

	-- Notify to services
	for _, app in pairs(manager._apps) do
		if application.is_running(app) and application.is_service(app) then
			application.handle_event(app, evt)
		end
	end

	if manager._app_in_control then
		result = true
		local app = getApp(manager, manager._app_in_control)
		if evt.type == "key" and (evt.value == KEY_EXIT or evt.value == KEY_ESCAPE) then
			application.close(app)
		else
			application.handle_event(app, evt)
		end
	end

	return result
end

function appmanagerDisableAll( manager )
	for _, app in pairs(manager._apps) do
		application.disable(app)
	end
end

----------------------------------------------------------------------------------
-- Application Manager Internals
----------------------------------------------------------------------------------

function registerApp( manager, file, enable )
	log.debug( "ApplicationManager", "try to register application defined in "  .. file)

	local envOrErr = sandbox.sandbox(file, config_env, file)
	local env = nil
	local errorMsg = nil

	if type(envOrErr) == "table" then
		env = envOrErr
	else
		errorMsg = envOrErr
	end

	result = false

	if env then
		if type(env.describe) == "function" then

			app = application.new(appID(file), env, manager, manager._surface)

			if application.initialize(app) then
				log.debug( "ApplicationManager", "register application defined in "  .. file)
				app.file = file

				manager._apps[app.id] = app

				if enable then
					result = application.enable(app)
				else
					result = true
				end
			else
				log.warn( "ApplicationManager", "ZaMBA application bad described: "  .. file)
			end
		else
			log.warn( "ApplicationManager", "ZaMBA application without describe method: " .. file)
		end
	else
		log.warn( "ApplicationManager", "ZaMBA application error: " .. errorMsg)
	end

	return result
end

function config_env( global_env, file )
	rootPath = util.splitPath(file)

	-- Canvas lib
	canvas = {}
	setmetatable(canvas, {__index = global_env.canvas})

	local createSurfaceFP = sandbox.wrap(global_env.canvas.createSurfaceFromPath, _G)

	canvas.createSurfaceFromPath = function(...)
		local s1 = createSurfaceFP(...)
		s1:setZIndex(canvas.zapperZIndex())
		return createSurfaceProxy(s1)
	end

	local createSurface = sandbox.wrap(global_env.canvas.createSurface, _G)
	canvas.createSurface = function(...)
		local s1 = createSurface(...)
		s1:setZIndex(canvas.zapperZIndex())
		return createSurfaceProxy(s1)
	end


	-- Timer lib
	_G.timer = {}

	function timer.register( ms )
		return global_env.mainWindow.registerTimer(ms)
	end

	function timer.cancel( timerID )
		global_env.mainWindow.cancelTimer(timerID)
	end

end

function appID( file )
	return string.gsub(file, "/", ".")
end

function start_window( manager, appID )
	pushCurrentWindow(zmb_apps.window)
	lockKeys()
	manager._app_in_control = appID
	canvas.flush()
end

function stop_window( manager )
	manager._surface:clear()
	canvas.flush()

	popCurrentWindow()
	unlockKeys()

	manager._app_in_control = nil
end

function is_running_interactive( manager )
	return manager._app_in_control ~= nil
end

function getApp( manager, appID )
	return manager._apps[appID]
end

function loadApps()
	return l_util.string.split(storage.loadStr("appManager.running_apps", ""), '\;')
end

----------------------------------------------------------------------------------
-- Aux functions
----------------------------------------------------------------------------------
function createSurfaceProxy( surface )
	local sp = proxy.createProxy(surface)
	sp.deny("autoFlush")
	return sp
end

