-------------------------------------------------------------------------------
-- app list
-------------------------------------------------------------------------------

tableOfApps = {}

function table.copy(t)
  local t2 = {}
  for k,v in pairs(t) do
    t2[k] = v
  end
  return t2
end

local application = require 'application'

function appListCreate( listName, x, y, w, h, showCount, colSizes )
	appList = {
		name=listName

		, visible_columns={"name", "interactive", "running"}
		, index_key="id"
		, headers={'Nombre', 'Tipo', 'Habilitada'}
		, columns_sizes= colSizes
		, show_count=showCount
		, lastAppSelected=0

		, list = list:new( x, y, w, h, menuColorTable )
	}

	list:setSelected( appList.list, 0)
	list:setVisibleRows(appList.list, appList.show_count);
	list:setCellWidths( appList.list, appList.columns_sizes);
	list:setCellHeaders(appList.list, appList.headers);

	return appList
end

function appListSelectApp( applist, app )
	list:setSelected( applist.list, app)
end

function appListSetApps( applist, apps )
	local i = 0
	local newApps = {}
	for _,app in pairs(apps) do
		tmpApp = table.copy(app)
		tmpApp.running = application.is_enable(tmpApp) and "Sí" or "No"
		tmpApp.interactive = application.is_interactive(tmpApp) and "Interactiva" or "Servicio"
		tableOfApps[i] = tmpApp
		table.insert(newApps,tmpApp)
		i = i + 1
	end
	list:setValues( applist.list, applist.visible_columns, applist.index_key, newApps )
end

function appListAddApps( applist, app )
	list:addRow( applist.list, applist.visible_columns, applist.index_key, app )
end

function appListUpdateApp( applist, app )
	list:updateRow( applist.list, list:selected( applist.list ), applist.visible_columns, applist.index_key, app )
end

function appListCurrentApp( applist )
	return list:getCurrentValue( applist.list )
end

function appListSelectedIndex( applist )
	return list:selected( applist.list )
end

function appListRowCount( appList )
	return list:getRowCount( appList.list )
end

-------------------------------------------------------------------------------
-- Apps list instance
-------------------------------------------------------------------------------

Apps_List_Window = {
	  name="Apps_List_Window"
	, ignoredKeys = {}
	, window = nil
	, help = "Solo las aplicaciones habilitadas pueden ser configuradas.\nNo todas las aplicaciones son configurables."
	, helpBox = nil
	, textarea
	, list = nil
	, label = nil
}

function Apps_List_Window.init()
	local window_x = safe_x+34
	local window_y = safe_y+10
	local window_w = canvas_w-2*(safe_x+34)
	local window_h = (canvas_h-2*(safe_y-1))-90

	Apps_List_Window.list = appListCreate( 'Channel_List_Standalone', safe_x+40, safe_y+40, canvas_w-2*(safe_x+40), 40, 10, {304,80,80} )
	Apps_List_Window.label = label:new(safe_x+40, safe_y+10, canvas_w-2*(safe_x+40), 0, menuColorTable, "Lista de canales - Sin canales", 16, Alignment.vTop_hCenter, 0)
	Apps_List_Window.window = window:new(window_x, window_y, window_w, window_h, appListZIndex, menuColorTable, 4)

	Apps_List_Window.helpBox = window:new(window_x, window_y + window_h + 5, window_w, 75, appListZIndex, menuColorTable, 4)
	Apps_List_Window.textarea = textarea:new(5, 5, window_w - 10, 60, menuColorTable, Apps_List_Window.help, 14, 3, 4)

	local buttonH = 42
	local buttonW = 98
	local buttonSep = 7

	Apps_List_Window.button_set = buttonset:new(((canvas_w-2*(safe_x+34)) / 2), ( safe_y+10 + canvas_h-2*(safe_y-1) ) - buttonH - buttonSep - 100, 308, buttonH, menuColorTable)
	buttonset:circular(Apps_List_Window.button_set, TRUE)

	btnEnable = button:new(0, 0, buttonW, buttonH, menuColorTable)
	activateEnableButton(true)
	button:selectedCallback(btnEnable, "appsOnEnableSelected")
	wgt:setHelp(     btnEnable, "\nOK = Habilitar     EXIT = Salir del menú")
	buttonset:addButton(Apps_List_Window.button_set, btnEnable)

	btnConfig = button:new(buttonW + 7, 0, buttonW, buttonH, menuColorTable)
	button:label(    btnConfig, "Configurar")
	button:icon(     btnConfig, "USB_Aplicaciones.png","USB_Aplicaciones.png", 24, 20)
	button:selectedCallback(btnConfig, "appsOnConfigureSelected")
	button:action(   btnConfig, "appsOnConfigure")
	wgt:setHelp(     btnConfig, "\nOK = Configurar     EXIT = Salir del menú")
	buttonset:addButton(Apps_List_Window.button_set, btnConfig)

	buttonset:setActiveButton(Apps_List_Window.button_set, 0)

	wgt:setVisible(Apps_List_Window.window, FALSE)
	wgt:setVisible(Apps_List_Window.helpBox, FALSE)

	window:addChild( Apps_List_Window.window, Apps_List_Window.list.list )
	window:addChild( Apps_List_Window.window, Apps_List_Window.label )
	window:addChild( Apps_List_Window.window, Apps_List_Window.button_set )
	window:addChild( Apps_List_Window.window, Apps_List_Window.helpBox )
	window:addChild( Apps_List_Window.helpBox, Apps_List_Window.textarea )

	wgt:fixOffsets(Apps_List_Window.button_set)
	wgt:fixOffsets(Apps_List_Window.helpBox)
end

function Apps_List_Window.close()
	hideAppList()
end

function appsOnEnable(b)
	if (appListRowCount(Apps_List_Window.list)>0) then
		Apps_List_Window.list.lastAppSelected = appListSelectedIndex( Apps_List_Window.list )
		local id = appListCurrentApp(Apps_List_Window.list)

		result = appmanagerEnableApp(manager, id)

		activateEnableButton(false)

		local app = getApp(manager, id)
		activateConfigButton(application.is_setupable(app) and application.is_enable(app))

		tableOfApps[Apps_List_Window.list.lastAppSelected].running =  result and "Sí" or "No"
		appListUpdateApp( Apps_List_Window.list, tableOfApps[Apps_List_Window.list.lastAppSelected] )
	end
end

function appsOnDisable(b)
	if (appListRowCount(Apps_List_Window.list)>0) then
		Apps_List_Window.list.lastAppSelected = appListSelectedIndex( Apps_List_Window.list )
		local id = appListCurrentApp(Apps_List_Window.list)

		appmanagerDisableApp(manager, id)
		
		activateEnableButton(true)
		activateConfigButton( false )
		
		tableOfApps[Apps_List_Window.list.lastAppSelected].running = "No"
		appListUpdateApp( Apps_List_Window.list, tableOfApps[Apps_List_Window.list.lastAppSelected] )
	end
end

function appsOnConfigureSelected(focus)
	if focus == TRUE then
		textarea:setText( Apps_List_Window.textarea, Apps_List_Window.help .. wgt:getHelp(btnConfig))
	end
end

function appsOnEnableSelected(focus)
	if focus == TRUE then
		textarea:setText( Apps_List_Window.textarea, Apps_List_Window.help .. wgt:getHelp(btnEnable))
	end
end

function openAppsList()
	pushCurrentWindow(Apps_List_Window)
	lockKeys()
	
	local apps = appmanagerList(manager)

	local cant = #apps
	local str = "Lista de Aplicaciones residentes - "
	if (cant>1) then
		str = str .. tostring(cant) .. " aplicaciones"
	elseif (cant==1) then
		str = str .. tostring(cant) .. " aplicación"
	else
		str = str .. "Sin aplicaciones"
	end
	label:setText( Apps_List_Window.label, str )
	wgt:open(Apps_List_Window.helpBox)
	wgt:open(Apps_List_Window.window)
	appListSetApps( Apps_List_Window.list, apps )
	appListSelectApp( Channel_List_Window.list, 0 )
	updateButtons()
end

function hideAppList()
	if (wgt:isVisible(Apps_List_Window.window) == TRUE) then
		wgt:close(Apps_List_Window.window)
		wgt:close(Apps_List_Window.helpBox)
		removeWindow('Apps_List_Window')
		unlockKeys()
	end
end

function Apps_List_Window.onExit()
	hideAppList()
	return true
end

function Apps_List_Window.onOk()
	if (appListRowCount(Apps_List_Window.list)>0) then
		Apps_List_Window.list.lastAppSelected = appListSelectedIndex( Apps_List_Window.list )
		appListSelectApp( Apps_List_Window.list, Apps_List_Window.list.lastAppSelected )
	end

	return true
end

function appsOnConfigure(b)
	if (appListRowCount(Apps_List_Window.list)>0) then
		Apps_List_Window.list.lastAppSelected = appListSelectedIndex( Apps_List_Window.list )
		appListSelectApp( Apps_List_Window.list, Apps_List_Window.list.lastAppSelected )
		local id = appListCurrentApp(Apps_List_Window.list)
		appmanagerStartSetupApp( manager, id)
	end
end

function updateButtons()
	if (appListRowCount(Apps_List_Window.list)>0) then
		local id = appListCurrentApp(Apps_List_Window.list)
		local app = getApp(manager, id)

		activateEnableButton(not application.is_enable(app))
		activateConfigButton(application.is_setupable(app) and application.is_enable(app))

		wgt:repaint(btnEnable)
		wgt:repaint(btnConfig)
		wgt:repaint(Apps_List_Window.button_set)
	end
end

function activateEnableButton( enable )
	local helpText = enable and "Habilitar" or "Deshabilitar"
	button:label(btnEnable, helpText)
	button:action(btnEnable, enable and "appsOnEnable" or "appsOnDisable")

	button:icon(btnEnable, "USB_Aplicaciones.png", "USB_Aplicaciones.png", 24, 20)
	wgt:setHelp(btnEnable, "\nOK = " .. helpText .. "     EXIT = Salir del menú")
	appsOnEnableSelected(TRUE)
end

function activateConfigButton( enable )
	buttonset:enableButton( Apps_List_Window.button_set, 1, enable and TRUE or FALSE)
end

setIgnoredKeys(Apps_List_Window,
             { KEY_MENU, KEY_SMALL_C
             , KEY_INFO, KEY_SMALL_I
             , KEY_GREEN, KEY_EPG, KEY_SMALL_E, KEY_SMALL_G
             , KEY_MUTE, KEY_SMALL_M
             , KEY_VOLUME_UP, KEY_SMALL_W
             , KEY_VOLUME_DOWN, KEY_SMALL_X
             , KEY_AUDIO, KEY_SMALL_A
             , KEY_SUBTITLE, KEY_SMALL_S
             , KEY_ASPECT, KEY_SMALL_A
             , KEY_CH34, KEY_SMALL_H
             , KEY_VIDEO, KEY_SMALL_V
             })

function Apps_List_Window.processKeys(key)
	if wgt:processKey(Apps_List_Window.window, key) == 0 then
		if wgt:processKey(Apps_List_Window.button_set, key) == 0 then
			return pkey(Apps_List_Window, key)
		end
	end
  
	if (key == KEY_CURSOR_DOWN) or (key == KEY_CURSOR_UP) then
		updateButtons()
	end
	return true
end

function Apps_List_Window.acceptsFocus()
	return true
end

Apps_List_Window.init()


return { openAppsList = openAppsList }
