-------------------------------------------------------------------------------
-- Generic channel list
-------------------------------------------------------------------------------

function chListCreate( listName, x, y, w, h, showCount, colSizes )
	chList = {
		name=listName

		, visible_columns={"channel","name", "favorite", "blocked"}
		, index_key="channelID"
		, headers={'Nro', 'Nombre del canal', 'Fav', 'Bloq'}
		, columns_sizes= colSizes
		, show_count=showCount
		, lastChannelSelected=0

		, list = list:new( x, y, w, h, menuColorTable )
	}

	list:setSelected( chList.list, 0)
	list:setVisibleRows(chList.list, chList.show_count);
	list:setCellWidths( chList.list, chList.columns_sizes);
	list:setCellHeaders(chList.list, chList.headers);

	list:mapColumn(chList.list, 2, "1", "img:Canales_Favorito_Blanco.png", "img:Canales_Favorito.png")
	list:mapColumn(chList.list, 3, "1", "img:Canales_Bloquear_Blanco.png", "img:Canales_Bloquear.png")

	return chList
end

function chListSelectChannel( chlist, channel )
	list:setSelected( chlist.list, channel)
end

function chListSetChannels( chlist, chs )
	list:setValues( chlist.list, chlist.visible_columns, chlist.index_key, chs )
end

function chListRefresh( chlist )
	list:updateScrollbar( chlist.list, TRUE )
end

function chListAddChannel( chlist, ch )
	list:addRow( chlist.list, chlist.visible_columns, chlist.index_key, ch )
end

function chListUpdateChannel( chlist, ch )
	list:updateRow( chlist.list, list:selected( chlist.list ), chlist.visible_columns, chlist.index_key, ch )
end

function chListCurrentChannel( chlist )
	return list:getCurrentValue( chlist.list )
end

function chListSelectedIndex( chlist )
	return list:selected( chlist.list )
end

function chListRowCount( chList )
	return list:getRowCount( chList.list )
end

-------------------------------------------------------------------------------
-- Channel list instance
-------------------------------------------------------------------------------

Channel_List_Window = {
	  name="Channel_List_Window"
	, ignoredKeys = {}
	, window = nil
	, list = nil
	, label = nil
}

function Channel_List_Window.init()
	Channel_List_Window.list = chListCreate( 'Channel_List_Standalone', safe_x+40, safe_y+40, canvas_w-2*(safe_x+40), canvas_h-2*(safe_y+40), 16, {61, 323, 40,40} )
	Channel_List_Window.label = label:new(safe_x+40, safe_y+10, canvas_w-2*(safe_x+40), 0, menuColorTable, "Lista de canales - Sin canales", 16, Alignment.vTop_hCenter, 0)
	Channel_List_Window.window = window:new(safe_x+34, safe_y+10, canvas_w-2*(safe_x+34), canvas_h-2*(safe_y-1), channelListZIndex, menuColorTable, 4)

	wgt:setVisible(Channel_List_Window.window, FALSE)
	window:addChild( Channel_List_Window.window, Channel_List_Window.list.list )
	window:addChild( Channel_List_Window.window, Channel_List_Window.label )
end

function Channel_List_Window.close()
	hideChannelList()
end

function openChannelList()
	pushCurrentWindow(Channel_List_Window)
	lockKeys()

	local cant = #channels.getAll()
	local str = "Lista de canales - "
	if (cant>1) then
		str = str .. tostring(cant) .. " canales"
	elseif (cant==1) then
		str = str .. tostring(cant) .. " canal"
	else
		str = str .. "Sin canales"
	end
	label:setText( Channel_List_Window.label, str )
	wgt:open(Channel_List_Window.window)
	local current = channel.current()
	local allChannels = channels.getAll()
	chListSetChannels( Channel_List_Window.list, allChannels )
	local idx=0
	for index,value in ipairs(allChannels) do
		if value.channelID == current then
			idx = index-1
			break
		end
	end
	chListSelectChannel( Channel_List_Window.list, idx )
end

function hideChannelList()
	if (wgt:isVisible(Channel_List_Window.window) == TRUE) then
		wgt:close(Channel_List_Window.window)
		chListSelectChannel(Channel_List_Window.list, -1)
		removeWindow('Channel_List_Window')
		unlockKeys()
	end
end

function Channel_List_Window.onChannelList()
	hideChannelList()
	return true
end

function Channel_List_Window.onRed()
	hideChannelList()
	return true
end

function Channel_List_Window.onExit()
	hideChannelList()
	return true
end

function Channel_List_Window.onOk()
	if (chListRowCount(Channel_List_Window.list)>0) then
		Channel_List_Window.list.lastChannelSelected = chListSelectedIndex( Channel_List_Window.list )
		chListSelectChannel( Channel_List_Window.list, Channel_List_Window.list.lastChannelSelected )
		local id = tonumber( chListCurrentChannel(Channel_List_Window.list) )
		hideChannelList()
		view.refresh()
		changeChannel(id, false)
	end

	return true
end

setIgnoredKeys(Channel_List_Window,
             { KEY_MENU, KEY_SMALL_C
             , KEY_INFO, KEY_SMALL_I
             , KEY_GREEN, KEY_EPG, KEY_SMALL_E, KEY_SMALL_G
             , KEY_MUTE, KEY_SMALL_M
             , KEY_VOLUME_UP, KEY_SMALL_W
             , KEY_VOLUME_DOWN, KEY_SMALL_X
             , KEY_AUDIO, KEY_SMALL_A
             , KEY_SUBTITLE, KEY_SMALL_S
             , KEY_ASPECT, KEY_SMALL_A
             , KEY_CH34, KEY_SMALL_H
             , KEY_VIDEO, KEY_SMALL_V
             , KEY_NUMBER_0, KEY_NUMBER_1
             , KEY_NUMBER_2, KEY_NUMBER_3
             , KEY_NUMBER_4, KEY_NUMBER_5
             , KEY_NUMBER_6, KEY_NUMBER_7
             , KEY_NUMBER_8, KEY_NUMBER_9
             })

function Channel_List_Window.processKeys(key)
	if wgt:processKey(Channel_List_Window.window, key) == 0 then
		return pkey(Channel_List_Window, key)
	end

	return true
end

function Channel_List_Window.acceptsFocus()
	return true
end

Channel_List_Window.init()
