local log = require 'log'
local VirtualChannel = -1
local RclChannel = -1

local ChannelByDigit = {}

local chColorTable = {
			bgLabelColor = Color.DarkBlue,
			textColor = Color.White,
			selectedTextColor = Color.Black
			}

ChannelByDigit.X = 548 --  X = 720-72-100 
ChannelByDigit.Y =  57
ChannelByDigit.W = 100
ChannelByDigit.H =  50
ChannelByDigit.BORDER_W = 4

local channelByDigit = window:new(ChannelByDigit.X
                                , ChannelByDigit.Y
                                , ChannelByDigit.W
                                , ChannelByDigit.H
				, widgetZIndex
				, menuColorTable
                                , ChannelByDigit.BORDER_W)
local maskedLabel = label:newMasked(ChannelByDigit.X +   ChannelByDigit.BORDER_W
                                  , ChannelByDigit.Y +   ChannelByDigit.BORDER_W
                                  , ChannelByDigit.W - 2*ChannelByDigit.BORDER_W
                                  , ChannelByDigit.H - 2*ChannelByDigit.BORDER_W
				  , chColorTable
                                  , ""
                                  , 28
                                  , Alignment.vTop_hCenter)

window:addChild(channelByDigit, maskedLabel)

wgt:setVisible(channelByDigit, FALSE)

function setVirtualChannel(c)
	VirtualChannel = c
end

local changeChannelDigitsCount = 0

function renderChannel()
	renderInfo(InfoMenu['visible'] == 2)
	if (Volume.mute == 1) then
		renderMute()
	end
end

function changeChannel(ix,isVirtual)
	if (ix ~= -1) then -- ix can be RclChannel, and it may be unset
		if (wgt:isVisible(Channel_List_Window.window) == TRUE) then
			Channel_List_Window.close()
		end
		if (channels.isProtected(ix) == TRUE) then
			changeChannelCallback = function() 
				renderZappingInfo(ix)
				doChangeChannel(ix, false)
			end
			PINDlg.init("Por favor, ingrese la clave", dialogZIndex)
			PINDlg.open(changeChannelCallback, "pinClose")
		else
			renderZappingInfo( ix )
			if (not isVirtual) then
				doChangeChannel( ix );
			else
				VirtualChannel = ix
			end
		end
	else
		log.info("Chs", "nextFav", "no favourite channel to change")
	end
end

function doChangeChannel(ix)
	RclChannel = channel.current()
	ignoreChKeys(true)
	if AppRunning.visible then
		hideApplicationRunning()
	end
	closeCC()
	view:refresh() -- in order to draw the zapping info before changing the channel
	channel.change(ix)
	VirtualChannel = -1

	zmb_apps.dispatchEvent({type="channelChanged", value=ix})
end

function changeChannelVirtual()
	changeChannel( VirtualChannel, false )
	VirtualChannel = -1
end

function getCurrentChannel( isVirtual )
	if (isVirtual and VirtualChannel ~= -1) then
		return VirtualChannel
	else
		return channel.current()
	end
end

function changeFactor( isVirtual, factor )
	local ch = channels.next( getCurrentChannel(isVirtual), factor )
	-- print('[lua] changeFactor: ch='..ch)
    changeChannel(ch,isVirtual)
end


function channelUp( isVirtual )
	changeFactor(isVirtual,1)
end

function channelDown( isVirtual )
	changeFactor(isVirtual,-1)
end

function previousChannel()
	-- print('[lua] previousChannel: ch='..RclChannel)
	changeChannel(RclChannel,false)
end

function nextFav()
    local nextChannel = channels.nextFavorite( channel.current(),1)
	changeChannel( nextChannel, false );
end

local ChangeChannelByDigitVisible = false
function closeChangeChannelByDigit()
	-- print('[lua] closeChangeChannelByDigit()')
	if ChangeChannelByDigitVisible then
		changeChannelDigitsCount = 0
		window:close(channelByDigit)
		label:reset(maskedLabel)
		ChangeChannelByDigitVisible = false
		closeChannelByDigitNav()
		mainWindow.cancelTimer(Timers.channelChange.id)
		Timers.channelChange.id = -1
	end
end

function changeChannelByDigits()
	-- print('[lua] changeChannelByDigits()')
	local str = label:getRawText(maskedLabel)
	closeChangeChannelByDigit()
	-- don't change to the same channel
	local cur = channel.current()
	if cur ~= -1 and channels.getInfo(cur).channel ~= str then
		local channels = channels.getAll()
		for i,ch in pairs(channels) do
			if (ch.channel == str) then
				changeChannel(ch.channelID,false)
			end
		end
	end
	checkUpdates()
	changeChannelDigitsCount = 0
end

function clearIncompleteChannel()
	closeChangeChannelByDigit()
end

function changeChannelByTimer()
	changeChannelByDigits( --[[digits]] )
end

function changeChannelPressDigit( key )
	if not ChangeChannelByDigitVisible then
		-- print('[lua] changeChannelPressDigit(), open')
		ChangeChannelByDigitVisible = true
--		label:setText(maskedLabel, key) -- TODO: See if we can fix it. This must be done here, manually becuse the window is not open yet and therefore it can't process the keys
		openChannelByDigitNav()
		ChannelByDigitNav.processKeys(key+1)
	end

	wgt:open(channelByDigit)
	changeChannelDigitsCount = changeChannelDigitsCount + 1

	if (changeChannelDigitsCount == 4) then
		changeChannelByDigits( --[[changeChannelDigits]] )
	else
		local tID = mainWindow.registerTimer(Timers['channelChange']['timeout'])
		Timers['channelChange']['id'] = tID
	end

end

function channelDeleted(chId)
	if RclChannel == chId then
		RclChannel = -1
	end
end

----------------    ChannelByDigitNav    ----------------
ChannelByDigitNav = {
	name = "ChannelByDigitNav"
	, ignoredKeys = {}
	, close = closeChangeChannelByDigit
}

setIgnoredKeys(ChannelByDigitNav, 
             { KEY_INFO, KEY_SMALL_I
             , KEY_GREEN, KEY_EPG, KEY_SMALL_E
             , KEY_CURSOR_DOWN
             , KEY_CURSOR_LEFT
             , KEY_MUTE, KEY_SMALL_M
             , KEY_VOLUME_UP, KEY_SMALL_W
             , KEY_VOLUME_DOWN, KEY_SMALL_X
             , KEY_MENU, KEY_SMALL_C
             , KEY_AUDIO, KEY_SMALL_A
             , KEY_CHANNEL, KEY_SMALL_P
             , KEY_FAVORITES, KEY_SMALL_F
             , KEY_SUBTITLE, KEY_SMALL_S
             , KEY_ASPECT, KEY_SMALL_A
             , KEY_CH34, KEY_SMALL_H
             , KEY_VIDEO, KEY_SMALL_V
             })

function ChannelByDigitNav.processKeys(key)
	-- print("[lua] ChannelByDigitNav.processKeys")
	if wgt:processKey(channelByDigit, key) == FALSE then
		return pkey(ChannelByDigitNav, key)
	end
	return true
end

function ChannelByDigitNav.onChannelUp()
	closeChangeChannelByDigit()
	channelUp( false )
	return true
end

function ChannelByDigitNav.onChannelDown()
	closeChangeChannelByDigit()
	channelDown( false )
	return true
end

function ChannelByDigitNav.onExit()
	closeChangeChannelByDigit()
	return true
end

function ChannelByDigitNav.acceptsFocus()
	return true
end

function openChannelByDigitNav()
	-- print('[lua] openChannelByDigitNav()')
	pushCurrentWindow(ChannelByDigitNav)
end

function closeChannelByDigitNav()
	-- print('[lua] closeChannelByDigitNav()')
	removeWindow('ChannelByDigitNav')
end



