TabWidth    =  95
TabHeight   =  65
TabSpace    =   5
PageX       = 182
PageY       =  67
PageW       = 456
PageH       = 345
PageMargin  =  17
PageBorder  =   4
HelpHeight  =  60

MainMenu = {
	  visible=false
	, x=safe_x+10, y=safe_y+10
	, width=canvas_w-2*(safe_x+10), height=TabHeight*5 + TabSpace*4
	, visible_columns={""}
	, title='Menú principal'
	, font_size=17
	, pages = {}
	, name = 'MainMenu'

	, index=0
	, show_from=0
	, show_to=4
	, show_count=5
	, scrollable=1
	, enabled=true
	, circular=true
	, ignoredKeys = {}
}

function openMainMenu()
	pushCurrentWindow(MainMenu)
	--print('[lua] openMainMenu(), lockKeys()')
	lockKeys()
	if(VolumeVisible) then
		VolumeVisible = false
	end
	tabbedwindow:selectTab(MainMenu.menu, 0)
	selectPage(MainMenu.currentPage(), true)
	wgt:open(MainMenu.logo)
	wgt:open(MainMenu.menu)
end

function hideMainMenu()
	if (wgt:isVisible(MainMenu.menu) == TRUE) then
		--print ('[lua] hideMainMenu')
		Page_PIN.labelsAreActive = false
		local page = MainMenu.currentPage()
		activatePage(page, false)
		selectPage(page, false)
		wgt:close(MainMenu.menu)
		wgt:close(MainMenu.logo)
		removeWindow('MainMenu')
		unlockKeys()

		checkUpdates()
	end
end

function MainMenu.close()
	hideMainMenu()
end

function MainMenu.processKeys(key)
	--print('[lua] MainMenu.processKeys('.. key ..')')
	local page = MainMenu.currentPage()
	if menupage:isActive(page.page)==TRUE and page.processKeys and page.processKeys(key) then
		--print('[lua] MainMenu, processed by page')
		return true
	end

	local ret = pkey(MainMenu, key)
	if not ret then
		ret = processKeysFrom(parent(MainMenu), key)
	end
	return ret
end

function MainMenu.acceptsFocus()
	return true
end

setIgnoredKeys(MainMenu, 
             { KEY_INFO, KEY_SMALL_I
             , KEY_GREEN, KEY_EPG, KEY_SMALL_E, KEY_SMALL_G
             , KEY_RED, KEY_SMALL_L, KEY_SMALL_R
             , KEY_MUTE, KEY_SMALL_M
             , KEY_VOLUME_UP, KEY_SMALL_W
             , KEY_VOLUME_DOWN, KEY_SMALL_X
             , KEY_AUDIO, KEY_SMALL_A
             , KEY_SUBTITLE, KEY_SMALL_S
             , KEY_ASPECT, KEY_SMALL_A
             , KEY_CH34, KEY_SMALL_H
             , KEY_VIDEO, KEY_SMALL_V
             })

function activatePage(page, state)
	if page.activate then
		page.activate(state)
	end
end

function selectPage(page, state)
	if page.select then
		MainMenu.updateHelp(page)
		page.select(state)
	end
	if state then
		wgt:setVisible(page.page, TRUE)
	else
		wgt:setVisible(page.page, FALSE)
	end
end

function enterMainMenu()
	local page = MainMenu.currentPage()
	activatePage(page, true)
	MainMenu.updateHelp(page)
end

function MainMenu.currentPage()
	 return MainMenu.pages[tabbedwindow:currentPageIndex(MainMenu.menu)]
end

function MainMenu.onRight()
	--print('[lua] MainMenu.rightArrow()')
	-- ask if the page is not active before entering the main menu
	-- to prevent entering the menu twice when none of the page's widgets handled the right arrow
	if (menupage:isActive(MainMenu.currentPage().page) == FALSE) then	
		enterMainMenu()
	end
	return true
end

function MainMenu.onOk()
	--print('[lua] MainMenu.ok()')
	enterMainMenu()
	return true
end

function MainMenu.onMenu()
	--print('[lua] MainMenu.onMenu()')
	hideMainMenu()
	return true
end

function MainMenu.onExit()
	--print('[lua] MainMenu.onExit()')
	local page = MainMenu.currentPage()
	if (menupage:isActive(page.page)==0) then
		hideMainMenu()
	else
		activatePage(page, false)
		MainMenu.updateHelp(page)
	end
	return true
end

function MainMenu.onUp()
	--print('[lua] MainMenu.up()')
	selectPage(MainMenu.currentPage(), false)
	tabbedwindow:selectPrevTab(MainMenu.menu)
	selectPage(MainMenu.currentPage(), true)
	return true
end

function MainMenu.onDown()
	--print('[lua] MainMenu.down()')
	selectPage(MainMenu.currentPage(), false)
	tabbedwindow:selectNextTab(MainMenu.menu)
	selectPage(MainMenu.currentPage(), true)
	return true
end


--function showButtons(selected, buttonsTable)
----
--end

require "page_channels"
require "page_usb"
require "page_config"
require "page_pin"
require "page_about"

function MainMenu.updateHelp(page)
	if wgt:isVisible(MainMenu.menu) == TRUE then
		menupage:setHelp(page.page, page.currentHelp())
	end
end

Channel_PageIx = 0
MMedia_PageIx  = 1
Config_PageIx  = 2
Clave_PageIx   = 3
About_PageIx   = 4

function MainMenu.init()
	menupage:setDefaultMargin(17)
	menupage:setDefaultBorder(4)
	menupage:setDefaultTabSpace(5)
	menupage:setDefaultTabWidth(95)
	menupage:setDefaultTabHeight(65)
	menupage:setDefaultHelpHeight(60)

	local helpHeight = 65

	MainMenu.menu = tabbedwindow:new(safe_x+10, safe_y+10, canvas_w-2*(safe_x+10), TabHeight*5 + TabSpace*4 + helpHeight, menuZIndex, menuColorTable)
	wgt:setVisible(MainMenu.menu, FALSE)

	MainMenu.pages[Channel_PageIx] = Page_Channels
	MainMenu.pages[MMedia_PageIx]  = Page_USB
	MainMenu.pages[Config_PageIx]  = Page_Config
	MainMenu.pages[Clave_PageIx]   = Page_PIN
	MainMenu.pages[About_PageIx]   = Page_About

	for k=0,4 do
		tabbedwindow:addPage(MainMenu.menu, MainMenu.pages[k].create())
	end

	MainMenu.logo = image:new(503, 40, 135, 23, backgroundZIndex, menuColorTable, "Logo.png")
	wgt:setVisible(MainMenu.logo, FALSE)

	wgt:fixOffsets(MainMenu.menu)
end

MainMenu.init()


