local log = require 'log'

----------------------------------------------------------------------------------
-- Root
-----------------------------------------------------------------------------------
RootNav = {
	  name = 'RootNav'
	, isRoot = true
	, stack_level = 1
	
	, IgnoreChKeys   = false
	, IgnoreAllKeys  = false
}

function ignoreChKeys(b)
	RootNav.IgnoreChKeys = b
end

function ignoreAllKeys(b)
	RootNav.IgnoreAllKeys = b
end

function canOpenZambaWidget()
	return (not ApplicationLaunching) and (ApplicationsRunning==0)
end

function RootNav.processKeys(key)
-- 	print('[lua] RootNav.processKeys('.. key ..')')
	return pkey(RootNav, key)
end

function RootNav.onMenu()
	if canOpenZambaWidget() then
		openMainMenu()
	end
	return true
end

function RootNav.onOk()
	if canOpenZambaWidget() then
		changeChannelVirtual()
	end
	return true
end

function RootNav.onVolUp()
	if isVolumeVisible() ~= true then
	end
	volumeUp()
	return true
end

function RootNav.onVolDown()
	if isVolumeVisible() ~= true then
	end
	volumeDown()
	return true
end

function RootNav.onDigit(digit)
	changeChannelPressDigit(digit)
	return true
end

function RootNav.onChannelDown()
	if (not RootNav.IgnoreChKeys) then
		channelDown( false )
	end
	return true
end

function RootNav.onChannelUp()
	if (not RootNav.IgnoreChKeys) then
		channelUp( false )
	end
	return true
end

function RootNav.onFav()
	if (not RootNav.IgnoreChKeys) then
		closeToRoot()
		nextFav()
	end
	return true
end

function RootNav.onRecall()
	if (not RootNav.IgnoreChKeys) then
		previousChannel()
	end
	return true
end

function RootNav.onDown()
	if canOpenZambaWidget() then
		channelDown( true )
	end
	return true
end

function RootNav.onUp()
	if canOpenZambaWidget() then
		channelUp( true )
	end
	return true
end

function RootNav.onMute()
	if canOpenZambaWidget() then
		toggleMute()
	end
	return true
end

function RootNav.onTVmode()
	if canOpenZambaWidget() then
		toggleVideoMode()
	end
	return true
end

function RootNav.onAspect()
	if canOpenZambaWidget() then
		toggleAspectMode()
	end
	return true
end

function RootNav.onSub()
	if canOpenZambaWidget() then
		toggleCaption()
	end
	return true
end

function RootNav.onAudio()
	if canOpenZambaWidget() then
		toggleAudio()
	end
	return true
end

function RootNav.onChannel()
	if canOpenZambaWidget() then
		toggleModulator()
	end
	return true
end

function RootNav.onInfo()
	if canOpenZambaWidget() then
		Info.onInfo()
	end
	return true
end

function RootNav.onEPG()
	if canOpenZambaWidget() and channels.haveChannels() ~= 0 then
		openEPG()
	end
	return true
end

function RootNav.acceptsFocus()
	return true
end

function RootNav.onGreen()
	--	Note: Used for ST to open EPG
	RootNav.onEPG()
end

function RootNav.onRed()
	--	Note: Used for ST to open channel list
	RootNav.onChannelList()
end

function RootNav.onChannelList()
	if (canOpenZambaWidget()) then
		openChannelList();
	end
	return true
end

function RootNav.onToggleFullscreen()
	local value = not mainWindow.isFullscreen()
	log.debug( "RootNav", "onToggleFullscreen", "fullscreen " .. tostring(value) )
	mainWindow.fullscreen(value)
	return true
end

--------------------------------------------------------------------

local WindowStack = {
	  elems = {RootNav}
	, size = 1
}

function _f(func)
	if (func) then
		func()
	end
end

function printWindowStack(prefix)
	for i=WindowStack.size,1,-1 do
		local window = WindowStack.elems[i]
		if window.name then
			--print('[lua] '.. prefix ..', window='..window.name)
		else
			--print('[lua] '.. prefix ..', window=?')
		end
	end
end

function removeWindow(windowName)
	if windowName == WindowStack.elems[WindowStack.size].name then --last Element
		popCurrentWindow()
	elseif windowName.stack_level == WindowStack.elems[1].name then --first Element
		moveElements(WindowStack, 1)
		WindowStack.size = WindowStack.size - 1
	else --in the midle
		position = findElement(windowName)
		if position ~= -1 then
			moveElements(WindowStack, position)
			WindowStack.size = WindowStack.size - 1
		end
	end
end

function moveElements(queue, position)
	currentPos = position
	while (currentPos < queue.size) do
		queue.elems[currentPos] = queue.elems[currentPos + 1]
		queue.elems[currentPos].stack_level = queue.elems[currentPos].stack_level - 1
		currentPos = currentPos + 1
	end
end

function findElement(name)
	position = 1
	while ( position <= WindowStack.size) and (name ~= WindowStack.elems[position].name) do
		position = position + 1
	end
	if position > WindowStack.size then
		return -1
	else
		return position
	end
end

function pushCurrentWindow(newWindow)
	--print('[lua] pushCurrentWindow '..WindowStack.size)

	WindowStack.size = WindowStack.size+1
	WindowStack.elems[WindowStack.size] = newWindow
	newWindow.stack_level = WindowStack.size

-- 	printWindowStack('pushCurrentWindow')
end

function popCurrentWindow()
	--print('[lua] popCurrentWindow '..WindowStack.size)

	WindowStack.elems[WindowStack.size].stack_level = -1
	WindowStack.elems[WindowStack.size] = nil
	WindowStack.size = WindowStack.size-1
	
-- 	printWindowStack('popCurrentWindow')
end

function CurrentWindow()
	return WindowStack.elems[WindowStack.size]
end

function existsFocusableWin()
	if WindowStack.size > 1 then
		for i=2, WindowStack.size do --Start from 2 to skip RootNav
			if WindowStack.elems[i].acceptsFocus() then
				return true
			end
		end
	end
	return false
end

function parent(window)
	if window.stack_level and window.stack_level>1 then
		return WindowStack.elems[window.stack_level-1]
	else
		return nil
	end
end

function closeCurrentWindow()
	--print('[lua] closeCurrentWindow()')
	_f(CurrentWindow().close)
end

function closeToRoot()
	for i=WindowStack.size,1,-1 do
		closeCurrentWindow()
	end
end

-----------------------------------------------------------------------------------

function getProcessKeys(window)
	if window and window ~= parent(window) then
		if window.acceptsFocus() ~= true then
			if parent(window) ~= nil then
				return getProcessKeys(parent(window))
			else
				return nil
			end
		else
			if window.processKeys then
				return window.processKeys
			else
				return getProcessKeys(parent(window))
			end
		end
	else
		return nil
	end
end

function processKeys(key)
	return processKeysFrom(CurrentWindow(), key)
end

function processKeysFrom(window, key)
	-- print('[lua] processKeysFrom('.. key ..')')
	local pKeys = getProcessKeys(window)
	if pKeys then
		return pKeys(key)
	else
		return false
	end
end

function setIgnoredKeys(nav, keys)
	for _,k in pairs(keys) do
		nav.ignoredKeys[k] = true
	end
end

function pkey(nav, key)
	local Actions = {
	  [KEY_INFO]         = 'onInfo'
	, [KEY_SMALL_I]      = 'onInfo'

	, [KEY_EPG]          = 'onEPG'
	, [KEY_SMALL_E]      = 'onEPG'

	, [KEY_CHANNEL_UP]   = 'onChannelUp'
	, [KEY_SMALL_Q]      = 'onChannelUp'
	, [KEY_CHANNEL_DOWN] = 'onChannelDown'
	, [KEY_SMALL_Z]      = 'onChannelDown'

	, [KEY_MUTE]         = 'onMute'
	, [KEY_SMALL_M]      = 'onMute'
	, [KEY_VOLUME_UP]    = 'onVolUp'
	, [KEY_SMALL_W]      = 'onVolUp'
	, [KEY_VOLUME_DOWN]  = 'onVolDown'
	, [KEY_SMALL_X]      = 'onVolDown'

	, [KEY_CURSOR_DOWN]  = 'onDown'
	, [KEY_CURSOR_LEFT]  = 'onLeft'
	, [KEY_CURSOR_RIGHT] = 'onRight'
	, [KEY_CURSOR_UP]    = 'onUp'
	, [KEY_OK]           = 'onOk'
	, [KEY_ENTER]        = 'onOk'

	, [KEY_RED]          = 'onRed'
	, [KEY_SMALL_R]      = 'onRed'
	, [KEY_GREEN]        = 'onGreen'
	, [KEY_SMALL_G]      = 'onGreen'
	, [KEY_YELLOW]       = 'onYellow'
	, [KEY_SMALL_Y]      = 'onYellow'
	, [KEY_BLUE]         = 'onBlue'
	, [KEY_SMALL_B]      = 'onBlue'

	, [KEY_NUMBER_0]     = 'onDigit'
	, [KEY_NUMBER_1]     = 'onDigit'
	, [KEY_NUMBER_2]     = 'onDigit'
	, [KEY_NUMBER_3]     = 'onDigit'
	, [KEY_NUMBER_4]     = 'onDigit'
	, [KEY_NUMBER_5]     = 'onDigit'
	, [KEY_NUMBER_6]     = 'onDigit'
	, [KEY_NUMBER_7]     = 'onDigit'
	, [KEY_NUMBER_8]     = 'onDigit'
	, [KEY_NUMBER_9]     = 'onDigit'
	
	, [KEY_MENU]         = 'onMenu'
	, [KEY_SMALL_C]      = 'onMenu'
	
	, [KEY_EXIT]         = 'onExit'
	, [KEY_ESCAPE]       = 'onExit'
	
	, [KEY_AUDIO]        = 'onAudio'
	, [KEY_SMALL_A]      = 'onAudio'
	
	, [KEY_CHANNEL]      = 'onRecall'
	, [KEY_SMALL_P]      = 'onRecall'
	
	, [KEY_FAVORITES]    = 'onFav'
	, [KEY_SMALL_F]      = 'onFav'
	
	, [KEY_SUBTITLE]     = 'onSub'
	, [KEY_SMALL_S]      = 'onSub'
	
	, [KEY_ASPECT]       = 'onAspect'
	, [KEY_SMALL_T]      = 'onAspect'
	
	, [KEY_CH34]         = 'onChannel'
	, [KEY_SMALL_H]      = 'onChannel'
	
	, [KEY_VIDEO]        = 'onTVmode'
	, [KEY_SMALL_V]      = 'onTVmode'

	, [KEY_SMALL_L]      = 'onChannelList'

	, [KEY_F11]          = 'onToggleFullscreen'

	}

	if Actions[key] then
		if nav.name then
			--print('[lua] pkey('.. nav.name .. ' ' .. key ..'), action = '..Actions[key])
		else
			--print('[lua] pkey('.. key ..'), action = '..Actions[key])
		end
		local action = nav[Actions[key]]
		if (nav.ignoredKeys and nav.ignoredKeys[key]) then
			--print('[lua] pkey, ignored')
			return true
		elseif action then
			log.debug( "RootNav", "pkey", "calling action " .. Actions[key])
			return action(key-1)
		else
			--print('[lua] pkey, processing parent')
			return processKeysFrom(parent(nav), key )
		end
	else
		-- print('[lua] pkey('.. key ..'), unrecognized')
	end
	return false
end

