local log = require 'log'
zmb_apps = require 'zmb_apps'

log.info("zamba", "init")

zmb_apps.enable(appsEnabled)

--display.setVideoMode( 2, 4 )

--Define bool constants
FALSE = 0
TRUE  = 1

--zindex values
backgroundZIndex = 1
sysZIndex = 2
muteZIndex = 2
widgetZIndex = 3
menuZIndex = 4
channelListZIndex = 4
appListZIndex = 4
dialogZIndex = 5

-- Cache fonts
canvas:cacheFonts( 'Tiresias' )

function round( val )
	return math.floor(val + 0.5)
end

canvas_w, canvas_h = canvas.size()
safe_x, safe_y, safe_w, safe_h = round(canvas_w/10), round(canvas_h/10), round(canvas_w/10*8), round(canvas_h/10*8)

---------- Key Locking ------------
local KeysLocked = false
function lockKeys()
	--print("[lua] lockKeys")
	if not KeysLocked and not CurrentWindow().isRoot then
		--print("[lua] locking")
		KeysLocked = true
		mainWindow.lockKeys(1)
	end
end

function unlockKeys()
	--print("[lua] unlockKeys")
	if KeysLocked and not existsFocusableWin() then
		--print("[lua] unlocking")
		KeysLocked = false
		mainWindow.lockKeys(0)
	end
end

--------------------------------------------------------------------------------------------------

require "colors"

require "nav"
require "chs"
require "volume"
require "download"
require "firmware_progress"
require "pin"
require "channel_list"
require "mainmenu"
require "parental"
require "toggles"
require "caption"

require "info"


--------------------------------------------------------------------------------------------------

ApplicationsRunning = 0
ApplicationLaunching = false

AppRunning = {
	  logo = nil
	, visible=false
	, visibleTime = 7000
}

function AppRunning.init()
	local img_w,img_h = 21,21
	local image = image:new(4,4, img_w,img_h, backgroundZIndex, menuColorTable, "i.png")
	AppRunning.logo = window:new( safe_x + safe_w - 2*img_w, safe_y + safe_h - (img_h+8)
	                        , img_w+8, img_h+8, sysZIndex
	                        , menuColorTable, 3)

	window:addChild(AppRunning.logo, image)
	wgt:fixOffsets(AppRunning.logo)
	
	wgt:setVisible(AppRunning.logo, FALSE)
	
	--view.setInteractivityIndicator(AppRunning.logo)
end

function showApplicationRunning()
	--print( "[lua] showApplicationRunning()")
	AppRunning.visible = true
	--view.showInteractivity(TRUE)
	view.refresh()
	local tID = mainWindow.registerTimer(Timers.appRunningTimer.timeout)
	Timers.appRunningTimer.id = tID
end

function hideApplicationRunning()
	--print( "[lua] hideApplicationRunning()")
	AppRunning.visible = false
	Timers.appRunningTimer.id = -1
	--view.showInteractivity(FALSE)
end

--------------------------------------------------------------------------------------------------
-- Zapper callbacks
--------------------------------------------------------------------------------------------------

function zapperApplicationLayerEnabled( isEnabled )
	--print( "[lua] Applications running:" .. isEnabled )
	if (isEnabled==1) then
		ApplicationsRunning = ApplicationsRunning + 1
		ApplicationLaunching = false
	else
		ApplicationsRunning = ApplicationsRunning - 1
	end

	if (ApplicationsRunning>0) then
		-- showApplicationRunning()
	else
		checkUpdates()
	end
end

function zapperApplicationDownloadProgress( step, total )
	win = CurrentWindow()
	if (win.name ~= 'MainMenu' and win.name ~= 'EPG') then
		local value = (step * 100) / total
		showDownloadApp( value )
	else
		clearDownloadApp()
	end
end

function zapperBeginScan( networks )
	Page_Channels.beginScan( networks )
end

function zapperScanNetwork( net )
	Page_Channels.scanNetwork( net )
	view.refresh()
end

function zapperEndScan()
	Page_Channels.endScan()
	if (channels.count() > 0) then
		hideMainMenu()
		changeChannel( 0, false )
	end
	view.refresh()
end

function zapperNewChannelFound( ch )
	Page_Channels.programFound( ch )
	view.refresh()
end

function zapperChannelUpdated( chID )
end

function zapperShowUpdated( chID, current )
	if EPG.visible then
		EPG.channelChanged(chID)
		view.refresh()
	end
	clearDownloadApp();
end

function zapperShowSubtitle( str )
	showCaption( str )
end

-------------------------------------------------------------------------------------
require "dialog"

-------------------------------------------------------------------------------------
require "firmupdate"

function zapperUpdateReceived( updateID )
	updateReceived(updateID)
	view.refresh()
end

function zapperUpdateDownloadProgress( step, total )
	win = CurrentWindow()
	if (win.name ~= 'MainMenu' and win.name ~= 'EPG') then
		local view = (step * 100) / total
		showDownloadFirmware( view )
	else
		clearDownloadFirmware()
	end
end

function zapperCurrentShowBlocked( blockedState )
	log.debug("zamba", "zapperCurrentShowBlocked", "blockedState=" .. tostring(blockedState))
	if blockedState and not PINDlg.opened then
		clearInfo()
		PINDlg.init("Ingrese la clave para ver los canales bloqueados", dialogZIndex)
		PINDlg.open(nil, "pinClose")
	elseif not blockedState and PINDlg.opened then
		pindialog:pinClose(PINDlg.dlg)
		view.refresh()
	end
end

-------------------------------------------------------------------------------------
local timerValue = 5000
local changeChannelTimeout = 5000

Timers = {
	  volume           = {id = -1, timeout=timerValue,           action=clearVolume}
	, channelChange    = {id = -1, timeout=changeChannelTimeout, action=changeChannelByTimer}
	, infoList         = {id = -1, timeout=timerValue,           action=clearInfo}
	, comboToggle      = {id = -1, timeout=timerValue,           action=clearComboToggle}
	, currentTime      = {id = -1, timeout=60*1000,              action=nil}
	, videoModeTimeOut = {id = -1, timeout=1000,                action=videoModeTick}
	, appRunningTimer  = {id = -1, timeout=AppRunning.visibleTime, action=hideApplicationRunning}
	, download  = {id = -1, timeout=3000, action=clearDownloadApp}
	, firmware  = {id = -1, timeout=3000, action=clearDownloadFirmware}
}

require "epg"

function zapperOnStandby( isOff )
	log.info("zamba", "On standby: off=" .. tostring(isOff) )
	if osOff == 1 then
		closeToRoot()
	end
end

function OnTimerEvent( tID )
-- 	print( "[lua] Timer " .. tID .. " expired" )
	found = false
	for k,timer in pairs(Timers) do
		if (tID == timer['id']) and timer.action then
			timer.action()
			found = true
			break
		end
	end
	if not found then
		zmb_apps.dispatchEvent({type="timeOut", value=tID})
	end
	view.refresh()
end

function OnKeyEvent( key, isUp )
	--print('[lua] OnKeyEvent, key == '.. key .. ' isUp == ' .. isUp )
	local result = zmb_apps.onKeyEvent(key, isUp)
	if not result then
		if isUp == 0 then
			if (not processKeys( key )) then
				if (key == KEY_EXIT or key == KEY_ESCAPE) then
					mainWindow.stop( 0 )
					return
				end
			end
			view.refresh()
		end
	end
end

function OnButtonEvent( id, isPress, x, y )
end

AppRunning.init()

if (channels.haveChannels() == 0) then
	openMainMenu()
	mainWindow.showBackground(1)
end
view.refresh()

if (channel.isBlocked()) then
	zapperCurrentShowBlocked(true)
end

zmb_apps.run()

mainWindow.run()

zmb_apps.fin()

log.info("zamba", "exit")
